from __future__ import annotations

import functools
import logging
import socket
from pathlib import Path
from typing import Any

import requests
import yaml

logging.basicConfig(
    level="INFO",
    format="%(asctime)s %(levelname)s %(filename)s %(message)s",
)
logger = logging.getLogger(__name__)


@functools.cache
def load_config(config_file: str | None = None) -> dict[str, Any] | None:
    filename = (
        config_file
        if config_file
        else Path(__file__).resolve().parent / "etc" / "settings.yaml"
    )
    logger.info("Loading config file from '%s'", filename)
    with open(filename) as stream:
        try:
            settings: dict[str, Any] = yaml.safe_load(stream)
        except yaml.YAMLError:
            logger.exception("Error parsing yaml file: '%s'", filename)
        else:
            return settings
    return None


def get_enabled_zone_names() -> set[str]:
    config = load_config()
    zone_names = [z['name'] for z in config.get('zones', [])]
    return set(get_zone_details(zone_names).keys())


def get_zone_names() -> set[str]:
    return set(get_zone_details().keys())


def get_zone_details(zones: list[str] | None = None) -> dict[str, Any]:
    def is_wanted_zone(zone: str) -> bool:
        return True if not zones else zone in zones

    config = load_config()
    url = f"{config['endpoint']}/zones"
    resp = requests.get(url, timeout=10)
    resp.raise_for_status()
    zones_json = resp.json()
    return {
        zone["roomName"]: zone
        for zone_json in zones_json
        for zone in zone_json["members"]
        if is_wanted_zone(zone["roomName"])
    }


def change_volume(increment: int) -> bool:
    config = load_config()
    for zone in get_enabled_zone_names():
        url_vol_increment: str = f"+{increment}" if increment > 0 else str(increment)
        url = f"""{config['endpoint']}/{zone}/volume/{url_vol_increment}"""
        logger.info(
            "Volume in zone: '%s' changing by %d.  Url is '%s'", zone, increment, url
        )
        resp = requests.get(url, timeout=10)
        resp.raise_for_status()
        resp_json = resp.json()
        if resp_json['status'] != 'success':
            logger.error("Response was not a success.  Received: %s", resp_json)
            return False
    return True


@functools.cache
def get_hostname(hostname: str) -> str:
    return socket.gethostbyname(hostname)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Callable

import time
from urllib.parse import parse_qs, urlparse

import flet as flt
from flet_timer.flet_timer import Timer
from typer import Typer

from sonos.utils import change_volume, get_hostname, get_zone_details, load_config

logger = logging.getLogger(__name__)

app = Typer()


def generate_volume_action(increment: int):
    def toggle_icon_button(e: flt.TapEvent) -> None:
        control: flt.Control = e.control
        control.selected = True
        control.update()
        time.sleep(0.2)
        change_volume(increment)
        control.selected = False
        control.update()

    return toggle_icon_button


def guess_artwork(config: dict, current_track: dict) -> str | None:
    album_art_uri = current_track.get("albumArtUri")

    if 'sonosradio' in album_art_uri:
        parsed_url = urlparse(album_art_uri)
        url = parse_qs(parsed_url.query)["mark"][0]
    else:
        hostname = config['zones'][0]['hostname']  # get the first controlled sonos zone
        logger.info("albumArtUri is '%s'", album_art_uri)
        url = f'http://{get_hostname(hostname)}:1400{album_art_uri}'

    logger.info("Will resolve from url '%s'", url)
    return url


def whats_playing(config: dict) -> dict:
    zones = [z["name"] for z in config["zones"]]
    if zones:
        details = get_zone_details(zones)
        if details:
            state = details[zones[0]]["state"]
            if "currentTrack" in state:
                return state["currentTrack"]
    return {}


def flet_app_updater(config_file: str | None = None) -> Callable[..., None]:
    config = load_config(config_file)

    def update_sonos_app(page: flt.Page) -> None:
        page.window.height = config["display"]["height"]
        page.window.width = config["display"]["width"]
        page.window.title_bar_hidden = True

        def refresh() -> flt.Container:
            track = whats_playing(config)
            artwork = guess_artwork(config, track)
            if page.controls:
                for control in page.controls:
                    if isinstance(control, flt.Container):
                        if artwork is None or control.image.src == artwork:
                            logger.warning("Artwork hasn't changed...")
                            return
                        page.controls.remove(control)
            container = flt.Container(
                image=flt.DecorationImage(
                    src=artwork,
                    fit=flt.ImageFit.COVER,
                ),
                expand=True,
                width=page.window.width,
                height=page.window.height,
                content=flt.Row(
                    [
                        flt.IconButton(
                            icon=flt.Icons.VOLUME_DOWN_ROUNDED,
                            on_click=generate_volume_action(-1),
                            selected=False,
                            style=flt.ButtonStyle(
                                color={
                                    "selected": flt.Colors.GREEN,
                                    "": flt.Colors.WHITE,
                                }
                            ),
                        ),
                        flt.IconButton(
                            icon=flt.Icons.VOLUME_UP_ROUNDED,
                            on_click=generate_volume_action(1),
                            selected=False,
                            style=flt.ButtonStyle(
                                color={
                                    "selected": flt.Colors.GREEN,
                                    "": flt.Colors.WHITE,
                                }
                            ),
                        ),
                    ],
                    vertical_alignment=flt.CrossAxisAlignment.END,
                    alignment=flt.MainAxisAlignment.SPACE_BETWEEN,
                ),
            )
            page.add(container)
            page.update()

        page.horizontal_alignment = flt.CrossAxisAlignment.CENTER
        page.vertical_alignment = flt.MainAxisAlignment.CENTER
        page.add(Timer(name="timer", interval_s=2, callback=refresh))
        page.update()

    return update_sonos_app


@app.command()
def run(config_file: str | None = None) -> None:
    flt.app(target=flet_app_updater(config_file))



# Always prefer setuptools over distutils
import pathlib

from setuptools import find_packages, setup

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

requirements = [
    'requests==2.32.3',
    'flet==0.25.2',
    'pyaml==24.12.1',
    'flet-timer==0.0.13',
    'typer==0.15.1',
]

setup(
    name="sonos-artwork",
    version="1.1.0",
    description="Displays Sonos Artwork playing in a zone",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/actionbronson/sonos-artwork",
    author="Philippe Legault",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3 :: Only",
    ],
    keywords="sonos",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.10, <4",
    install_requires=requirements,
    extras_require={
        "dev": ["pytest", "black", "ruff", "twine"],
    },
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "sonos=sonos.main:app",
        ],
    },
)
